#include "PullMoveSet.hh"
#include "PivotMoveSet.hh"
#include "LatticeDescriptorCUB.hh"
#include "LatticeModel.hh"
#include "LatticeProtein.hh"
#include <iostream>

using namespace biu;

void print3DData(LatticeProtein* lp)
{
	std::cout<<"->printing coordinates:" << std::endl;
	DPointVec vec = lp->get3Ddata();
	for (DPointVec::const_iterator it = vec.begin();  
	it != vec.end(); it++) 
	{
		std::cout << "  "<<  *it << std::endl;
	}	
}

void testPullMoveDecoderNeighborhood()
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : PullMoveSet::PullMoveDecoder\n"
				<<"===========================================" <<std::endl;
	
	LatticeDescriptorCUB * ldcub = new LatticeDescriptorCUB();
	LatticeModel * lattice = new LatticeModel(ldcub);
	std::cout 	<< "=> creating PullMoveSet::PullMoveDecoder(new Lattice" 
				<< std::endl;
	PullMoveSet::PullMoveDecoder * decoder = new PullMoveSet::PullMoveDecoder(
			lattice);
	
	const LatticeNeighborhood neighbors = lattice->getNeighborhood();
	// for every possivle direction vector
	for (LatticeNeighborhood::const_iterator it = neighbors.begin();
			it != neighbors.end(); it++) {
		std::cout << "direction vector: " << (*it) << std::endl;
		// get every set of C and CL
		for (size_t i=0; i<4; i++) {
			IPointVec lcvec = decoder->lookupCL(IntPoint(0,0,0), (*it), i);
			std::cout 	<< "  Move: " << i << std::endl;
			std::cout 	<< "    C: " << lcvec[0] << std::endl;
			std::cout 	<< "    L: " << lcvec[1] << std::endl;
			// check validity
			std::cout 	<< "    C and L neighbored? " 
						<< lattice->areNeighbored(lcvec[0], lcvec[1]) 
						<< std::endl;
			std::cout 	<< "    C is neighbored to (0,0,0)? " 
						<< lattice->areNeighbored(IntPoint(0,0,0), lcvec[0]) 
						<< std::endl;
			std::cout 	<< "    L is neighbored to fixed direction? " 
						<< lattice->areNeighbored((*it), lcvec[1]) 
						<< std::endl;
		}
	}

	delete decoder;
	delete ldcub;
	delete lattice;
}

void testPullMoveDecoderLookup()
{
	std::cout << "=> creating helper objects" << std::endl;
	LatticeDescriptorCUB ld;
	LatticeModel lm(&ld);
	Alphabet alph("HP",1);
	EnergyMatrix E = EnergyMatrix(	0.0, 
			alph.getAlphabetSize(),
			alph.getAlphabetSize());
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	ContactEnergyFunction f(&alph, &E);
	
	std::cout 	<< "=> creating PullMoveSet(new LatticeModel(new LatticeDescriptorCUB()))" 
				<< std::endl;
	PullMoveSet pms(&lm);
	
	std::string seqStr = "PHPHHHH";
	std::cout	<<"=> "<<seqStr <<" = seqStr" <<std::endl;
	std::string absMoveStr = "FFFFFF";
	std::cout	<<"=> "<<absMoveStr <<"  = absMoveStr" <<std::endl;
	std::cout	<<"=> creating biu::LatticeProtein(lm,f,seqStr,absMoveStr,true)"
	<<" = lp" 
	<<std::endl;
	LatticeProtein* lp = new LatticeProtein(&lm,&f,seqStr,absMoveStr,true);
	std::cout	<< "->getAbsoluteMoveString = " 
				<< lp->getAbsoluteMoveString() << std::endl;
	std::cout	<< "->getRelativeMoveString = " 
				<< lp->getRelativeMoveString() << std::endl;
	std::cout	<< "pms.getMoveNumber(lp) = "
				<< pms.getMoveNumber(lp) << std::endl;
	
	bool pullFront;
	bool stdPull;
	size_t moveIndex;
	size_t movePosition;
	size_t size = lp->getSequence().size();
	
	for (size_t i=0; i<pms.getMoveNumber(lp); i++)
	{
		moveIndex = i;
		std::cout 	<< "pms->decoder.lookupMove(lp.getSequence().size(), " 
		<< i
		<< ",pullFront,stdPull,movePosition)"
		<< std::endl;
		pms.decoder->lookupMove(size, moveIndex,pullFront,stdPull,movePosition);

		std::cout	<< "  pullFront = " << (pullFront ? "true" : "false")
		<< std::endl;
		std::cout	<< "  stdPull = " << (stdPull ? "true" : "false")
		<< std::endl;
		if (!stdPull)
		{
			IPointVec pullMoves = pms.decoder->lookupEndMove(IntPoint(0,0,0), moveIndex);
			std::cout 	<< "  endPullPos0 = " 
						<< pullMoves.at(0)
						<< std::endl;
			std::cout 	<< "  endPullPos1 = " 
						<< pullMoves.at(1)
						<< std::endl;
						
		}
		std::cout	<< "  moveIndex = " << moveIndex
		<< std::endl;
		std::cout	<< "  movePosition = " << movePosition
		<< std::endl;
	}
	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
}

void testPullMoveDecoderEndMove()
{
	
}

void testPullMoveSet_a()
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
	<<" test function : PullMoveSet\n"
	<<"===========================================" <<std::endl;

	std::cout << "=> creating helper objects" << std::endl;
	LatticeDescriptorCUB ld;
	LatticeModel lm(&ld);
	Alphabet alph("HP",1);
	EnergyMatrix E = EnergyMatrix(	0.0, 
			alph.getAlphabetSize(),
			alph.getAlphabetSize());
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	ContactEnergyFunction f(&alph, &E);

	std::cout 	<< "=> creating PullMoveSet(new LatticeModel(new LatticeDescriptorCUB()))" 
	<< std::endl;
	PullMoveSet pms(&lm);

	/*
	 *		Test 1:
	 */ 
	std::string seqStr = "PHPHHHH";
	std::cout	<<"=> "<<seqStr <<" = seqStr" <<std::endl;
	std::string absMoveStr = "FFFFFF";
	std::cout	<<"=> "<<absMoveStr <<"  = absMoveStr" <<std::endl;
	std::cout	<<"=> creating biu::LatticeProtein(lm,f,seqStr,absMoveStr,true)"
				<<" = lp" 
				<<std::endl;
	LatticeProtein* lp = new LatticeProtein(&lm,&f,seqStr,absMoveStr,true);
	std::cout	<< "->getAbsoluteMoveString = " 
				<< lp->getAbsoluteMoveString() << std::endl;
	std::cout	<< "->getRelativeMoveString = " 
				<< lp->getRelativeMoveString() << std::endl;
	std::cout	<< "pms.getMoveNumber(lp) = "
				<< pms.getMoveNumber(lp) << std::endl;
	
	// applying moves
	for (size_t i=0; i<pms.getMoveNumber(lp); i++)
	{
		std::cout 	<< "pms.applyMoveInPlace(lp,0," << i << ") = "
		<< pms.applyMoveInPlace(lp,0,i)->getAbsoluteMoveString() 
		<< std::endl;
		std::cout 	<< "pms.undoLastMove(lp)->getAbsoluteMoveString() = "
		<< pms.undoLastMove(lp)->getAbsoluteMoveString()
		<< std::endl;
	}
	
//	size_t i = 0;
//	std::cout 	<< "pms.applyMoveInPlace(lp,0," << i << ") = "
//			<< pms.applyMoveInPlace(lp,0,i)->getAbsoluteMoveString() 
//			<< std::endl;

	delete lp;
}

void testPullMoveSet_b()
{
	std::cout << "=> creating helper objects" << std::endl;
	LatticeDescriptorCUB ld;
	LatticeModel lm(&ld);
	Alphabet alph("HP",1);
	EnergyMatrix E = EnergyMatrix(	0.0, 
			alph.getAlphabetSize(),
			alph.getAlphabetSize());
	E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	ContactEnergyFunction f(&alph, &E);

	std::cout 	<< "=> creating PullMoveSet(new LatticeModel(new LatticeDescriptorCUB()))" 
	<< std::endl;
	PullMoveSet pms(&lm);
	
	//	/*
	//	 *		Test 2:
	//	 */
	std::string seqStr2 = "PHPHHH";
	std::cout	<<"=> "<<seqStr2 <<" = seqStr2" <<std::endl;
	std::string absMoveStr2 = "RULUU";
	std::cout	<<"=> "<<absMoveStr2 <<"  = absMoveStr2" <<std::endl;
	std::cout	<<"=> creating biu::LatticeProtein(lm,f,seqStr2,absMoveStr2,true)"
	<<" = lp" 
	<<std::endl;
	LatticeProtein* lp = new LatticeProtein(&lm,&f,seqStr2,absMoveStr2,true);
	std::cout<< "->getAbsoluteMoveString = " << lp->getAbsoluteMoveString() << std::endl;
	std::cout<< "->getRelativeMoveString = " << lp->getRelativeMoveString() << std::endl;
		
	// applying moves
	for (size_t i=0; i<pms.getMoveNumber(lp); i++)
	{
		std::cout 	<< "pms.applyMoveInPlace(lp,0," << i << ") = "
		<< pms.applyMoveInPlace(lp,0,i)->getAbsoluteMoveString() 
		<< std::endl;
		std::cout 	<< "pms.undoLastMove(lp)->getAbsoluteMoveString() = "
		<< pms.undoLastMove(lp)->getAbsoluteMoveString()
		<< std::endl;
	}
		
		delete lp;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
}

int main(int argc, char **argv) 
{
	testPullMoveDecoderNeighborhood();
	testPullMoveDecoderLookup();
	testPullMoveDecoderEndMove();
 	
	testPullMoveSet_a();
	testPullMoveSet_b();
	
	return 0;
}

